-----------------------------------------------------------------------------------------------
-- Client Lua Script for RepTrak (Silk3 Edit)
-- Copyright (c) NCsoft. All rights reserved
-- Reputation Tracker Coded by Kardall (Silk3 Edit)
-- 	With Help from Lua Natics, Togglebutton, Sinaloit, Daihenka
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "Sound"

-----------------------------------------------------------------------------------------------
-- RepTrak Module Definition
-----------------------------------------------------------------------------------------------
local RepTrak = {}
local wndToggle = true
local tFactionList = {}
local karRepToColor = 
{
	ApolloColor.new("ff9aaea3"),
	ApolloColor.new("ff9aaea3"), -- Neutral
	ApolloColor.new("ff836725"), -- Liked
	ApolloColor.new("ffc1963d"), -- Accepted
	ApolloColor.new("fff1efda"), -- Popular
	ApolloColor.new("fffefbb5"), -- Esteemed
	ApolloColor.new("ffd5b66d"), -- Beloved
}
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-------------------------------------------------------------------------------------------------
function RepTrak:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function RepTrak:Init()
    Apollo.RegisterAddon(self)
end
 

-----------------------------------------------------------------------------------------------
-- RepTrak OnLoad
-----------------------------------------------------------------------------------------------
function RepTrak:OnLoad()
    -- Register handlers for events, slash commands and timer, etc.
    -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
    Apollo.RegisterSlashCommand("reptrak", "OnRepTrakOn", self)
    Apollo.RegisterEventHandler("ReputationChanged", "OnReputationChanged", self)
    -- load our forms
    self.wndMain = Apollo.LoadForm("RepTrak.xml", "RepTrakForm", nil, self)
    self.wndMain:Show(false)
	self.tReputationLevels = GameLib.GetReputationLevels()
	self.bFirstLoad = true
	self.storage = ""
	-- Grab the First Faction
	local tReputations = GameLib.GetReputationInfo()
	for idx, tRep in pairs(tReputations) do
		Event_FireGenericEvent("SendVarToRover", "tReputations", tReputations)
		if not tRep.bIsLabel then
			self.tReputationLevels = GameLib.GetReputationLevels()
			local tLevelData = self.tReputationLevels[tRep.nLevel]
			local fBar = self.wndMain:FindChild("FillBar")
			fBar:SetMax(tLevelData.nMax)
			fBar:SetProgress(tRep.nCurrent)
			fBar:SetBarColor(karRepToColor[tRep.nLevel + 1])
			fBar:EnableGlow(tRep.nCurrent > tLevelData.nMin)
			fBar:SetText(String_GetWeaselString(Apollo.GetString("Achievements_ProgressBarProgress"), tRep.nCurrent, tLevelData.nMax))
		end 
	end 
end


-----------------------------------------------------------------------------------------------
-- RepTrak Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/reptrack"
function RepTrak:OnRepTrakOn()
	if self.bFirstLoad then
		
		self.wndMain:FindChild("FillBar"):SetMax(0)
		self.wndMain:FindChild("FillBar"):SetProgress(0)
		self.wndMain:FindChild("FillBar"):SetText(String_GetWeaselString(Apollo.GetString("Achievements_ProgressBarProgress"), 0, 0))
		
		self.bFirstLoad = false
	end	
	self.wndMain:Show(wndToggle)
	wndToggle = (not wndToggle)
	self.factionList = {}
	Sound.Play(Sound.PlayUIAlertLeveledUp)
	self:PopulatedDDList()
end

function RepTrak:CreateDropDown()
	local theOne = ""
	self:PopulatedDDList()
	self.DDMenu = Apollo.LoadForm("RepTrak.xml", "DropDownMenu", self.wndMain, self)
	for idx = 1, #self.factionList do
		local wndCurr = Apollo.LoadForm("RepTrak.xml", "DDButton", self.DDMenu, self)
		wndCurr:SetText(self.factionList[idx])
		wndCurr:SetData(self.factionList[idx])
		theOne = self.factionList[idx]
	end
	self.DDMenu:ArrangeChildrenVert(0)
	local x,y = Apollo.GetScreenSize()
	local wndPar = self.wndMain:FindChild("EventType")
	local nL,nT,nR,nB = self.wndMain:GetAnchorOffsets() -- Only care about nT
	local nParL, nParT, nParR, nParB = wndPar:GetAnchorOffsets()
	local nOffL, nOffT, nOffR, nOffB = self.DDMenu:GetAnchorOffsets()
	local screenMiddle = y/2 
	if nT > screenMiddle then 
		-- higher than half way, grow menu up
		nParB = nParB - ( (#self.factionList + 1) * 24)
	end
	self.DDMenu:SetAnchorOffsets(nParL, nParB, (nParL + nOffR), (nParB + ( #self.factionList * 24 )) )
	self.DDMenu:ToFront()
end

function RepTrak:OnDDClick()
	if self.DDMenu and self.DDMenu:IsShown() == false then
		self:CreateDropDown()
		self.DDMenu:Show(true)
		self.DDMenu:ToFront()
	elseif self.DDMenu and self.DDMenu:IsShown() == true then
		self.DDMenu:Show(false)
	elseif not self.DDMenu then
		self:CreateDropDown()
	end
end

function RepTrak:DDButtonClick(wndHandler, wndControl)
	local strEventType = wndControl:GetText()
	self.wndMain:FindChild("EventType"):FindChild("Button"):FindChild("Text"):SetText(strEventType)
	self.storage = "" -- TODO: Hack to make announcing work. Breaks if you select a different one, then the same one you had.
	self:OnReputationChanged(wndControl:GetText())
	self.DDMenu:Show(false)
end

function RepTrak:OnReputationChanged(tFaction) -- tFaction is the TEXT of the actual Faction (the one you see in the rep screen)
	local tReputations = GameLib.GetReputationInfo()
	if not tReputations then
		return
	end
	self.tReputationLevels = GameLib.GetReputationLevels()
	local objFactionName = self.wndMain:FindChild("EventType"):FindChild("Button"):FindChild("Text")
	local objFactionProgressBar = self.wndMain:FindChild("FillBar")
	for idx, tRep in pairs(tReputations) do
		-- this is going to cycle through each reputation for the character
		-- Eliminate the labels first
		if not tRep.bIsLabel then
			-- It's not a label lets see if it matches what came in
			if string.lower(tRep.strName) == string.lower(objFactionName:GetText()) then
				local tLevelData = self.tReputationLevels[tRep.nLevel]
				objFactionProgressBar:SetMax(tLevelData.nMax)
				objFactionProgressBar:SetProgress(tRep.nCurrent)
				objFactionProgressBar:SetBarColor(karRepToColor[tRep.nLevel + 1])
				if self.storage == "" then
					-- No Data To Know, Set Old Color
					self.storage = karRepToColor[tRep.nLevel + 1]
				else 
					if self.storage ~= karRepToColor[tRep.nLevel + 1] then
						-- color has changed since our last update. Obviously dinged reps
						Sound.Play(Sound.PlayUIAlertPopUpMessageReceived)
					else
						-- set old color for next time
						self.storage = karRepToColor[tRep.nLevel + 1]
					end 
				end
				-- Set the % in bar
				local percent = (tonumber(tRep.nCurrent) / tonumber(tLevelData.nMax)) * 100
				if percent > 100 then percent = 100 
					self.wndMain:FindChild("txtPercent"):SetText( string.format("%.0f %%", percent))
				else 
					self.wndMain:FindChild("txtPercent"):SetText( string.format("%.1f %%", percent))
				end 
				objFactionProgressBar:EnableGlow(tRep.nCurrent > tLevelData.nMin)
				objFactionProgressBar:SetText(String_GetWeaselString(Apollo.GetString("Achievements_ProgressBarProgress"), tRep.nCurrent, tLevelData.nMax))
								
			end
		end
	end		
end

-- Drop Down Menu Functions

function RepTrak:PopulatedDDList()
	-- Generate List Of Items To Display
	local tDDList = {}
	
	-- List of Player Reputations
	local tCharacterRep = GameLib.GetReputationInfo()
	Event_FireGenericEvent("SendVarToRover", "tCharacterRep", tCharacterRep)
	
	-- Loop Through Ignoring Label Reps
	for k, v in pairs(tCharacterRep) do
		if not v.bIsLabel then
			table.insert(tDDList, v.strName)
			table.sort(tDDList)
		end
	end
	Event_FireGenericEvent("SendVarToRover", "tDDList", tDDList)
	
	self.factionList = tDDList
end



-----------------------------------------------------------------------------------------------
-- RepTrakForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function RepTrak:OnOK()
	self.wndMain:Show(false) -- hide the window
end

-- when the Cancel button is clicked
function RepTrak:OnCancel()
	self.wndMain:Show(false) -- hide the window
end


-----------------------------------------------------------------------------------------------
-- RepTrak Instance
-----------------------------------------------------------------------------------------------
local RepTrakInst = RepTrak:new()
RepTrakInst:Init()
